/*
 * Copyright (c) 2020-2026 Valve Corporation
 * Copyright (c) 2020-2026 LunarG, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Authors:
 * - Christophe Riccio <christophe@lunarg.com>
 */

#pragma once

#include <vector>
#include <string>

enum LogType {
    LOG_NONE = 0,
    LOG_ERROR,
    LOG_WARN,
    LOG_INFO,
    LOG_DEBUG,
    LOG_PERF,
    LOG_LAYER,
    LOG_DRIVER,

    LOG_FIRST = LOG_ERROR,
    LOG_LAST = LOG_DRIVER,

    LOG_MESSAGE_INVALID = ~0,
};
enum { LOG_COUNT = LOG_LAST - LOG_FIRST + 1 };

enum LogBit {};
typedef int LogFlags;

const char* GetToken(LogType value);

LogType GetLogType(const char* token);

const char* GetToken(LogBit bit);

LogBit GetLogBit(const char* token);

LogBit GetBit(LogType type);

LogType GetValue(LogBit bit);

LogFlags GetLogFlags(const std::vector<std::string>& tokens);

std::vector<std::string> GetLogTokens(LogFlags flags);

LogFlags GetLogFlags(const std::string& string);

std::string GetLogString(LogFlags flags);
